@@ -33,24 +33,34 @@ The main interface is :class:`PrestoQuery`: ::
     >> rows = list(query.execute())
 """
 
-from urllib.parse import urlunparse
 import logging
+import os
+from requests_kerberos.exceptions import KerberosExchangeError
 from typing import Any, Dict, List, Optional, Text, Tuple, Union  # NOQA for mypy types
 
 import requests
 
 from prestodb import constants
 from prestodb import exceptions
+import prestodb.redirect
 
 
 __all__ = ['PrestoQuery', 'PrestoRequest']
 
 
 logger = logging.getLogger(__name__)
+logger.setLevel(logging.INFO)
 
 
 MAX_ATTEMPTS = constants.DEFAULT_MAX_ATTEMPTS
-
+SOCKS_PROXY = os.environ.get('SOCKS_PROXY')
+if SOCKS_PROXY:
+    PROXIES = {
+        'http': 'socks5://' + SOCKS_PROXY,
+        'https': 'socks5://' + SOCKS_PROXY,
+    }
+else:
+    PROXIES = None
 
 class ClientSession(object):
     def __init__(
@@ -191,6 +201,7 @@ class PrestoRequest(object):
         http_headers=None,  # type: Optional[Dict[Text, Text]]
         http_scheme=constants.HTTP,  # type: Text
         auth=constants.DEFAULT_AUTH,  # type: Optional[Any]
+        redirect_handler=prestodb.redirect.GatewayRedirectHandler(),
         max_attempts=MAX_ATTEMPTS,  # type: int
         request_timeout=constants.DEFAULT_REQUEST_TIMEOUT,  # type: Union[float, Tuple[float, float]]
         handle_retry=exceptions.RetryWithExponentialBackoff(),
@@ -214,8 +225,9 @@ class PrestoRequest(object):
         if self._auth:
             if http_scheme == constants.HTTP:
                 raise ValueError('cannot use authentication with HTTP')
-            self._auth.set_session(self._session)
+            self._auth.set_http_session(self._http_session)
 
+        self._redirect_handler = redirect_handler
         self._request_timeout = request_timeout
         self._handle_retry = handle_retry
         self.max_attempts = max_attempts
@@ -239,11 +251,8 @@ class PrestoRequest(object):
 
         # merge custom http headers
         for key in self._client_session.headers:
-            if key.startswith(constants.HEADER_PREFIX):
-                raise ValueError('invalid HTTP header {}: must not start with "{}"'.format(
-                    key,
-                    constants.HEADER_PREFIX,
-                ))
+            if key in headers.keys():
+                raise ValueError('cannot override reserved HTTP header {}'.format(key))
         headers.update(self._client_session.headers)
 
         return headers
@@ -266,10 +275,14 @@ class PrestoRequest(object):
         with_retry = exceptions.retry_with(
             self._handle_retry,
             exceptions=(
-                exceptions.Http503Error,
+                KerberosExchangeError,
                 PrestoRequest.http.ConnectionError,
                 PrestoRequest.http.Timeout,
             ),
+            conditions=(
+                # need retry when there is no exception but the status code is 503
+                lambda response: getattr(response, 'status_code', None) == 503,
+            ),
             max_attempts=self._max_attempts,
         )
         self._get = with_retry(self._http_session.get)
@@ -278,8 +291,12 @@ class PrestoRequest(object):
 
     def get_url(self, path):
         # type: Text -> Text
-        destination = '{}:{}'.format(self._host, self._port)
-        return urlunparse((self._http_scheme, destination, path, '', '', ''))
+        return "{protocol}://{host}:{port}{path}".format(
+            protocol=self._http_scheme,
+            host=self._host,
+            port=self._port,
+            path=path
+        )
 
     @property
     def statement_url(self):
@@ -292,24 +309,49 @@ class PrestoRequest(object):
         return self._next_uri
 
     def post(self, sql):
-        return self._post(
+        data = sql.encode('utf-8')
+        http_headers = self.http_headers
+
+        http_response = self._post(
             self.statement_url,
-            data=sql.encode('utf-8'),
-            headers=self.http_headers,
+            data=data,
+            headers=http_headers,
             timeout=self._request_timeout,
+            allow_redirects=self._redirect_handler is None,
+            proxies=PROXIES,
         )
+        if self._redirect_handler is not None:
+            while http_response is not None and http_response.is_redirect:
+                location = http_response.headers['Location']
+                url = self._redirect_handler.handle(location)
+                logger.info('redirect {} from {} to {}'.format(
+                    http_response.status_code,
+                    location,
+                    url,
+                ))
+                http_response = self._post(
+                    url,
+                    data=data,
+                    headers=http_headers,
+                    timeout=self._request_timeout,
+                    allow_redirects=False,
+                    proxies=PROXIES,
+                )
+        return http_response
 
     def get(self, url):
         return self._get(
             url,
             headers=self.http_headers,
             timeout=self._request_timeout,
+            proxies=PROXIES,
         )
 
     def delete(self, url):
         return self._delete(
             url,
             timeout=self._request_timeout,
+            proxies=PROXIES,
         )
 
     def _process_error(self, error):
@@ -323,12 +365,12 @@ class PrestoRequest(object):
 
     def raise_response_error(self, http_response):
         if http_response.status_code == 503:
-            raise exceptions.Http503Error('service unavailable')
+            raise exceptions.Http503Error('error 503: service unavailable')
 
         raise exceptions.HttpError(
-            'error {}: {}'.format(
+            'error {}{}'.format(
                 http_response.status_code,
-                http_response.content,
+                ': {}'.format(http_response.content) if http_response.content else '',
             )
         )
 
@@ -336,7 +378,7 @@ class PrestoRequest(object):
         # type: requests.Response -> PrestoStatus:
         status_code = http_response.status_code
         # mypy cannot follow module import
-        if status_code != self.http.codes.ok:  # type: ignore
+        if not http_response.ok:
             self.raise_response_error(http_response)
 
         http_response.encoding = 'utf-8'
@@ -418,8 +460,10 @@ class PrestoQuery(object):
         self._columns = None
 
         self._finished = False
+        self._cancelled = False
         self._request = request
         self._sql = sql
+        self._result = PrestoResult(self)
 
     @property
     def columns(self):
@@ -429,6 +473,10 @@ class PrestoQuery(object):
     def stats(self):
         return self._stats
 
+    @property
+    def result(self):
+        return self._result
+
     def execute(self):
         # type: () -> PrestoResult
         """Initiate a Presto query by sending the SQL statement
@@ -438,6 +486,8 @@ class PrestoQuery(object):
         track the rows returned by the query. To fetch all rows,
         call fetch() until is_finished is true.
         """
+        if self._cancelled:
+            raise exceptions.PrestoUserError("Query has been cancelled")
 
         response = self._request.post(self._sql)
         status = self._request.process(response)
@@ -446,8 +496,8 @@ class PrestoQuery(object):
         self._stats.update(status.stats)
         if status.next_uri is None:
             self._finished = True
-        self.result = PrestoResult(self, status.rows)
-        return self.result
+        self._result = PrestoResult(self, status.rows)
+        return self._result
 
     def fetch(self):
         # type: () -> List[List[Any]]
@@ -467,6 +517,11 @@ class PrestoQuery(object):
         # type: None -> None
         if self.is_finished():
             return
+
+        self._cancelled = True
+        if self._request.next_uri is None:
+            return
+
         response = self._request.delete(self._request.next_uri)
         if response.status_code == requests.codes.no_content:
             return
